


ADC Project - with ATmega32


CC Dharmani          www.dharmanitech.com

*********************************

Hi friends,

Im presenting here a small project on how to make use of the ADC of ATmega32. The circuit checks the voltage present at the ADC channel and displays voltage value on the LCD (16x2). Whenever the push-button is pressed, the LCD displays voltage present on the next ADC channel (total 8 channels, CH0 to CH7). The schematic and the firmware of the project are given here.


Code:


Total 6 files: 3 program files (ADC_main.c, LCD_routines.c, ADC_routines.c) and 3 header files(ADC_main.h, LCD_routines.h, ADC_routines.h)


//*****************************************************************
// **************** ADC_routines.h *********************
//*****************************************************************

#ifndef ADC_ROUTINES_H

#define ADC_ENABLE ADCSRA |= (1<<ADEN)
#define ADC_DISABLE ADCSRA &= 0x7F
#define ADC_START_CONVERSION ADCSRA |= (1<<ADSC)

void ADC_init(void);
int ADC_read(void);
float ADC_calculateTemp(int);
unsigned char* updateTempDisplay(float);
unsigned char* temporary(void);
void ADC_displayValue(void);

#endif


//*****************************************************************
// **************** ADC_routines.c *********************
//*****************************************************************
//Controller: ATmega32 (16 MHz)
//Compiler: ICCAVR
//Author: CC Dharmani, Chennai
//Date: Aug 2008
//*****************************************************************

#include <iom32v.h>
#include <macros.h>
#include "ADC_routines.h"
#include "LCD_routines.h"

unsigned char valueDisplay[]=": . volt"; //11 char global string for

//voltage value display

float Vref = 5.00; // Reference voltage Vref of ADC

//*****************************************************************
//Purpose : Initialize the ADC
//*****************************************************************
void ADC_init(void)
{
ADCSRA = 0x00; //disable adc
ADMUX = 0x40; //select adc input 0
ADCSRA = 0x86;
}

/********************************************************************
* Purpose : Do a Analog to Digital Conversion
* Paramtr : none
* return : integer voltage value
********************************************************************/
int ADC_read(void)
{
char i;
int ADC_temp, ADCH_temp;
int ADC_var = 0;

ADC_ENABLE;
ADC_START_CONVERSION; //do a dummy readout first
while(!(ADCSRA & 0x10)); // wait for conversion done, ADIF flag active
ADCSRA|=(1<<ADIF);

for(i=0;i<8;i++) // do the ADC conversion 8 times for better accuracy
{
ADC_START_CONVERSION;
while(!(ADCSRA & 0x10)); // wait for conversion done, ADIF flag set
ADCSRA|=(1<<ADIF);

ADC_temp = ADCL; // read out ADCL register
ADCH_temp = ADCH; // read out ADCH register
ADC_temp +=(ADCH_temp << 8);
ADC_var += ADC_temp; // accumulate result (8 samples)

//for later averaging
}

ADC_var = ADC_var >> 3; // average the 8 samples
ADC_DISABLE;
return ADC_var;
}

/**************************************************************************
* Purpose : To calculate temp in degre celsius
* Paramtr : Integer value of voltage, received from ADC read
* Return : float voltage value
***************************************************************************/
float ADC_calculateValue(int inputValue)
{
float actualValue;

actualValue=(inputValue * Vref/1024.0); //calculates the voltage present
return actualValue;
}

/***************************************************************************
* Purpose : To update the tempDisplay string based on the latest temp read
* Paramtr : Float value of voltage
* Return : String pointer pointing to the updated valueDisplay
***************************************************************************/
unsigned char* updateDisplay(float actualValue)
{
int temp;
unsigned char c;
temp=(int)(actualValue*100.0); //to include decimal point for display

if((actualValue*100.0 - temp) >= 0.5) temp=temp+1;

valueDisplay[5] = ((unsigned char)(temp%10)) | 0x30;
temp=temp/10;
valueDisplay[4] = ((unsigned char)(temp%10)) | 0x30;
temp=temp/10;
valueDisplay[2] = ((unsigned char)(temp%10)) | 0x30;
temp=temp/10;

return valueDisplay;

}

/**************************************************************************
* Purpose : To display value of voltage applied to selected ADC channel on

the LCD
* Paramtr : None
* Return : None
***************************************************************************/
void ADC_displayValue(void)
{
int value;
float value1;

value = ADC_read();
value1 = ADC_calculateValue(value);

LCD_DisplayString(2,4,updateDisplay(value1)); //voltage display on 2nd row,

//4th column
}

//*****************************************************************
// **************** LCD_routines.c *********************
//*****************************************************************
#ifndef LCD_ROUTINES_H

// *** LCD Functions declaration *** //
void LCD_init(void);
void LCD_WriteCommand (unsigned char CMD);
void LCD_WriteData (unsigned char Data);

void LCD_DisplayString_F(char row, char column, const unsigned char *string);
void LCD_DisplayString(char row, char column, unsigned char *string);
void LCD_Cursor(char row, char column);
void delay_ms(int miliSec);

#define ENABLE_LCD PORTD |= 0x80
#define DISABLE_LCD PORTD &= ~0x80

#define SET_LCD_DATA PORTD |= 0x20
#define SET_LCD_CMD PORTD &= ~0x20

#endif


//********************************************************
// ************** LCD_routines.c ******************
//********************************************************
//Controller: ATmega32 (16 MHz)
//Compiler: ICCAVR
//Author: CC Dharmani, Chennai
//Date: Aug 2008
//********************************************************

#include <iom32v.h>
#include <macros.h>
#include "LCD_routines.h"

// *********************************
// *** Initialize the LCD driver ***
// *********************************
void LCD_init(void)
{
delay_ms(100); // wait for 100ms

LCD_WriteCommand (0x38); // 8 data lines
LCD_WriteCommand (0x06); // cursor setting
LCD_WriteCommand (0x0f); // display ON
LCD_WriteCommand (0x01); // clear LCD memory
delay_ms (10); // 10ms delay after clearing LCD
}

// **********************************************
// *** Write a command instruction to the LCD ***
// **********************************************
void LCD_WriteCommand (unsigned char Command)
{

SET_LCD_CMD; // Set LCD in command mode

PORTC = Command; // Load data to port

ENABLE_LCD; // Write data to LCD
asm("nop");
asm("nop");
DISABLE_LCD; // Disable LCD

delay_ms(1); // wait for 1ms
}

// *****************************************
// *** Write one byte of data to the LCD ***
// *****************************************
void LCD_WriteData (unsigned char Data)
{
SET_LCD_DATA; // Set LCD in data mode

PORTC = Data; // Load data to port

ENABLE_LCD; // Write data to LCD
asm("nop");
asm("nop");
DISABLE_LCD; // Disable LCD

delay_ms(1); // wait for 1ms
}

// *********************************************************************
// *** Display a string at the specified row and column, from FLASH ****
// *********************************************************************
void LCD_DisplayString_F ( char row, char column , const unsigned char *string)
{
LCD_Cursor (row, column);
while (*string)
LCD_WriteData(*string++);
}

// *********************************************************************
// *** Display a string at the specified row and column, from RAM ****
// *********************************************************************
void LCD_DisplayString (char row, char column , unsigned char *string)
{
LCD_Cursor (row, column);
while (*string)
LCD_WriteData(*string++);
}

// ***************************************************
// *** Position the LCD cursor at "row", "column". ***
// ***************************************************
void LCD_Cursor (char row, char column)
{
switch (row)
{
case 1: LCD_WriteCommand (0x80 + column - 1); break;
case 2: LCD_WriteCommand (0xc0 + column - 1); break;
default: break;
}
}

// ********************************************************
// **** Function for delay of 1 msec (appx.) at 16Mhz *****
// ********************************************************
void delay_ms(int miliSec) //for 16 Mhz crystal
{
int i,j;

for(i=0;i<miliSec;i++)
for(j=0;j<1550;j++)
{
asm("nop");
asm("nop");
}
}

//********************************************************************
// ADC_main.h
//********************************************************************


#ifndef ADC_MAIN_H

#define KEY_PRESSED !(PINB & 0x02)
#define KEY_OPEN (PINB & 0x02)

void port_init(void);
void init_devices(void);

#endif


//********************************************************************
// ****** MAIN PROGRAM FOR TESTING ADC OF MEGA32  ADC_main.c******
//********************************************************************
//Controller: ATmega32 (Crystal: 16Mhz)
//Compiler: ICCAVR
//Author: CC Dharmani, Chennai
//Date: Aug 2008
//********************************************************************

#include <iom32v.h>
#include <macros.h>
#include "ADC_main.h"
#include "LCD_routines.h"
#include "ADC_routines.h"

//port initialize
void port_init(void)
{
DDRA = 0x00;
PORTA = 0x00;
DDRB = 0x00;
PORTB = 0x02; //internal pull-up for push-button
DDRC = 0xFF;
PORTC = 0x00;
DDRD = 0xF0;
PORTD = 0x00;
}

//call this routine to initialize all peripherals
void init_devices(void)
{
//stop errant interrupts until set up
CLI(); //disable all interrupts

port_init();
ADC_init();
LCD_init();

MCUCR = 0x00;
TIMSK = 0x00; //timer interrupt sources
// SEI(); //re-enable interrupts
}

// MAIN FUNCTION

void main(void)
{
unsigned char channel = 0;

init_devices();

LCD_DisplayString_F(1,1,"ADC Testing.. ");

while(1)
{
LCD_DisplayString_F(2,1,"CH");
LCD_WriteData(channel | 0x30); //display channel number

START:

while(KEY_OPEN) //wait here till key is open
{
ADMUX &= 0xe0;
ADMUX |= channel; //select ADC channel
ADC_displayValue();
delay_ms(50);
}

delay_ms(20); //key debounce delay when key is pressed
if(KEY_OPEN)
goto START;

while(KEY_PRESSED); //wait here till key is pressed
delay_ms(20); //key debounce delay when key is released

channel++; //select next channel after key-press
if(channel == 8) //only channel-0 to channel-7 can be selected
channel = 0;
}

}

//*********************** END **************************


CC Dharmani                                                                              www.dharmanitech.com


˹ҹ (ҤسѺԷ) |
Google Documents   ҹʹ ôյҹ

 
